/**
 * Copyright (C) 2016 OGIS-RI Co., Ltd.
 * 
 * This software is released under the MIT License.
 * http://opensource.org/licenses/mit-license.php
 */

var express = require('express');
var path = require('path');
var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');

var routes = require('./routes/index');

var passport = require('passport');
var OpenidConnectStrategy = require('passport-openidconnect').Strategy;

var app = express();
app.use(passport.initialize());
process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');

// uncomment after placing your favicon in /public
//app.use(favicon(path.join(__dirname, 'public', 'favicon.ico')));
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

app.use('/', routes);

// catch 404 and forward to error handler
app.use(function(req, res, next) {
  var err = new Error('Not Found');
  err.status = 404;
  next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
  app.use(function(err, req, res, next) {
    res.status(err.status || 500);
    res.render('error', {
      message: err.message,
      error: err
    });
  });
}

// production error handler
// no stacktraces leaked to user
app.use(function(err, req, res, next) {
  res.status(err.status || 500);
  res.render('error', {
    message: err.message,
    error: {}
  });
});

const ISSUER_IDENTIFIER = "https://openidprovider.sample.domain:443/openam/oauth2";
const AUTHORIZATION_ENDPOINT = "https://openidprovider.sample.domain:443/openam/oauth2/authorize";
const TOKEN_ENDPOINT = "https://openidprovider.sample.domain:443/openam/oauth2/access_token";
const USERINFO_ENDPOINT = "https://openidprovider.sample.domain:443/openam/oauth2/userinfo";
const CLIENT_ID = "sampleRP";
const CLIENT_SECRET = "password";
const CALLBACK = "https://relyingparty.sample.domain/cb";
const SCOPE = ["profile", "email"];
const ALGORITHM = "RS256";
const MODULUS = "";// 公開鍵のnの値
const EXPONENT = "";// 公開鍵のeの値

passport.use(new OpenidConnectStrategy(
  {
    authorizationURL: AUTHORIZATION_ENDPOINT,
    tokenURL: TOKEN_ENDPOINT,
    userInfoURL: USERINFO_ENDPOINT,
    clientID: CLIENT_ID,
    clientSecret: CLIENT_SECRET,
    callbackURL: CALLBACK,
    scope: SCOPE
  }, function(iss, sub, profile, jwtClaims, accessToken, refreshToken, params, done) {
    // 署名アルゴリズムの検証
    // ID Tokenのヘッダに含まれる署名アルゴリズムが、定義したALGORITHM値と一致することを検証する
    var headerStr = new Buffer(params.id_token.split(".")[0], "base64").toString();
    var header = JSON.parse(headerStr);
    if (header.alg !== ALGORITHM) {
      done("Missmatch Algorithm", null);
    }

    // issクレームの検証
    // issクレームの値が、定義したISSUER_IDENTIFIERと一致することを検証する
    if (jwtClaims.iss !== ISSUER_IDENTIFIER) {
      done("Missmatch Issuer", null);
    }

    // audクレームの検証
    // audクレームの値が、定義したCLIENT_IDが含まれることを検証する
    if (jwtClaims.aud.indexOf(CLIENT_ID) < 0) {
      done("Missmatch Audience", null);
    }

    // azpクレームの検証
    // audクレームの値が複数ある場合に、azpクレームが、定義したCLIENT_IDと一致することを検証する
    if (jwtClaims.aud.split(",").length > 1 &&
        (!jwtClaims.claims.azp ||
        jwtClaims.claims.azp !== CLIENT_ID)) {
      done("Missmatch Authorized party", null);
    }

    // expクレームの検証
    // expクレームの値が、現在時刻よりも大きいることを検証する
    var now = Math.round(Date.now()/1000);
    if (jwtClaims.exp <= now) {
      done("Expired token", null);
    }

    // iatクレームの検証
    // iatクレームの値が、現在時刻から一時間以内かどうかを検証する
    var ONE_HOUR = 3600;
    var now = Math.round(Date.now()/1000);
    if (jwtClaims.iat + ONE_HOUR <= now) {
      done("Expired token", null);
    }

    // ID Tokenの署名検証
    // ID Tokenの署名が正しいかどうか検証する
    var getPem = require('rsa-pem-from-mod-exp');
    var pem = getPem(MODULUS, EXPONENT);
    var token = params.id_token;

    var jwt = require('jsonwebtoken');
    jwt.verify(token, pem, function(err, decoded) {
      if (err) {
        done("Invalid Signature", null);
      } else {
        done(null, profile, params);
      }
    });
  }
));

passport.serializeUser(function(user, done){
  done(null, user);
});

passport.deserializeUser(function(obj, done){
  done(null, obj);
});

module.exports = app;
